package com.bipt.intelligentapplicationorchestrationservice.service.Impl;

import com.bipt.intelligentapplicationorchestrationservice.constant.StatusConstant;
import com.bipt.intelligentapplicationorchestrationservice.enumeration.DatasetType;
import com.bipt.intelligentapplicationorchestrationservice.mapper.DatasetMapper;
import com.bipt.intelligentapplicationorchestrationservice.pojo.*;
import com.bipt.intelligentapplicationorchestrationservice.service.DatasetService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import static com.bipt.intelligentapplicationorchestrationservice.enumeration.DatasetType.FROM_DATABASE;
import static com.bipt.intelligentapplicationorchestrationservice.enumeration.DatasetType.UPLOAD;

@Service
@Slf4j
public class DatasetServiceImpl implements DatasetService {
    @Autowired
    private DatasetMapper datasetMapper;

    @Override
    @Transactional
    public void save(DatasetDTO datasetDTO, MultipartFile file) {
        DatasetType datasetType = DatasetType.fromCode(datasetDTO.getDatasetType());
        if (datasetType == UPLOAD && file != null && !file.isEmpty()) {
            try {
                //todo存入分布式文件系统

                // 生成文件名（建议使用 UUID 避免重复）
                String fileName = UUID.randomUUID() + "-" + file.getOriginalFilename();
                // 假设使用本地存储（实际需对接分布式文件系统，如 MinIO、OSS 等）
                String filePath = "/your/storage/path/" + fileName;
                // 保存文件到磁盘（示例代码，需处理 IO 异常）
                file.transferTo(new File(filePath));
                // 更新数据集路径
                datasetDTO.setDsPath(filePath); // 取消注释，并确保 datasetDTO 有 setDsPath 方法
            } catch (IOException e) {
                throw new RuntimeException("文件上传失败: " + e.getMessage());
            }
        } else if (datasetType == FROM_DATABASE){
            //todo 从数据仓库中查询 并选择筛选条件
        }

        // 保存数据集实体
        DatasetEntity datasetEntity = new DatasetEntity();
        BeanUtils.copyProperties(datasetDTO, datasetEntity);
        datasetEntity.setCreateTime(LocalDateTime.now());
        datasetEntity.setUpdateTime(LocalDateTime.now());
        datasetMapper.insert(datasetEntity);
    }


    /**
     * 修改数据集
     *
     * @param datasetDTO
     * @param file
     */
    @Override
    @Transactional
    public void update(DatasetDTO datasetDTO, MultipartFile file) {
        DatasetType datasetType = DatasetType.fromCode(datasetDTO.getDatasetType());
        if (datasetType == UPLOAD && file != null && !file.isEmpty()) {
            try {
                // 生成文件名（建议使用 UUID 避免重复）
                String fileName = UUID.randomUUID() + "-" + file.getOriginalFilename();
                // 假设使用本地存储（实际需对接分布式文件系统，如 MinIO、OSS 等）
                String filePath = "/your/storage/path/" + fileName;
                // 保存文件到磁盘（示例代码，需处理 IO 异常）
                file.transferTo(new File(filePath));
                // 更新数据集路径
                datasetDTO.setDsPath(filePath); // 取消注释，并确保 datasetDTO 有 setDsPath 方法
            } catch (IOException e) {
                throw new RuntimeException("文件上传失败: " + e.getMessage());
            }
        } else if (datasetType == FROM_DATABASE){
            //todo 从数据仓库中查询 并选择筛选条件
        }

        DatasetEntity datasetEntity = new DatasetEntity();
        BeanUtils.copyProperties(datasetDTO,datasetEntity);
        datasetEntity.setUpdateTime(LocalDateTime.now());
        datasetMapper.updata(datasetEntity);
    }

    /**
     * 分页查询
     * @param dataSetPageQueryDTO
     * @return
     */
    @Override
    public PageResult pageQuery(DatasetPageQueryDTO dataSetPageQueryDTO) {
        PageHelper.startPage(dataSetPageQueryDTO.getPage(), dataSetPageQueryDTO.getPageSize());
        Page<DatasetVO> page = datasetMapper.pageQuery(dataSetPageQueryDTO);
        return new PageResult(page.getTotal(), page.getResult());
    }

    /**
     * 批量删除
     * @param datasetIds
     */
    @Override
    @Transactional
    public void deleteBatch(List<Long> datasetIds) {

        //TODO 在分布式文件系统中删除
        datasetMapper.deleteBatch(datasetIds);
    }

    /**
     * 下载
     * @param datasetId
     * @return
     */
    @Override
    public InputStream downloadDataset(Long datasetId) {
        // TODO: 调用分布式存储系统的接口获取数据集文件的输入流
        return null;
    }
}
